"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

require("dom-renderer/dist/polyfill");

var _domRenderer = require("dom-renderer");

var _fsExtra = require("fs-extra");

var _path = require("path");

var _amdBundle = _interopRequireDefault(require("amd-bundle"));

var Parser = _interopRequireWildcard(require("./parser"));

var _utility = require("./utility");

var _nodeToolkit = require("@tech_query/node-toolkit");

/**
 * Component packer
 */
class Component {
  /**
   * @param {string} path - Component directory
   */
  constructor(path) {
    this.path = path;
    this.name = (0, _path.basename)(path);
    this.entry = (0, _path.join)(path, 'index');
  }
  /**
   * @param {string} path
   *
   * @return {DocumentFragment}
   */


  static parseHTML(path) {
    return (0, _asyncToGenerator2.default)(function* () {
      const box = document.createElement('div'),
            fragment = document.createDocumentFragment();
      box.innerHTML = (yield (0, _fsExtra.readFile)(path)) + '';
      fragment.append(...box.childNodes);
      return fragment;
    })();
  }
  /**
   * @param {string}  source - File path or Style source code
   * @param {?string} type   - MIME type
   * @param {string}  [base] - Path of the file which `@import` located in
   *
   * @return {?Element} Style element
   */


  static parseCSS(source, type, base) {
    return (0, _asyncToGenerator2.default)(function* () {
      var style;
      type = type ? type.split('/')[1] : (0, _path.extname)(source).slice(1);
      if (!source.includes('\n')) source = (yield (0, _fsExtra.readFile)(base = source)) + '';
      style = Parser[type] && (yield Parser[type](source, (0, _path.dirname)(base)));
      return style && Object.assign(document.createElement('style'), {
        textContent: style
      });
    })();
  }
  /**
   * @param {DocumentFragment} fragment
   *
   * @return {Element[]}
   */


  static findStyle(fragment) {
    return [].concat.apply([], Array.from(fragment.querySelectorAll('link[rel="stylesheet"], template'), tag => tag.content ? Array.from(tag.content.querySelectorAll('style')) : tag));
  }
  /**
   * @param {string} tagName
   *
   * @return {string}
   */


  static identifierOf(tagName) {
    return tagName[0].toUpperCase() + tagName.replace(/-(\w)/g, (_, char) => char.toUpperCase()).slice(1);
  }
  /**
   * @param {String} path - Full name of a JS file
   *
   * @return {String} Packed JS source code
   */


  static packJS(path) {
    const single_entry = (0, _path.join)((0, _utility.folderOf)().lib || '', 'index.js');
    const name = path === single_entry && (0, _utility.metaOf)().name;
    path = path.split('.').slice(0, -1).join('.');
    return new _amdBundle.default(path).bundle(name || (0, _path.basename)((0, _path.dirname)(path)));
  }
  /**
   * @return {DocumentFragment} HTML version bundle of this component
   */


  toHTML() {
    var _this = this;

    return (0, _asyncToGenerator2.default)(function* () {
      const fragment = yield Component.parseHTML(_this.entry + '.html'),
            CSS = [];
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = Component.findStyle(fragment)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          let sheet = _step.value;
          let style = yield Component.parseCSS(sheet.tagName === 'STYLE' ? sheet.textContent : (0, _path.join)(_this.path, sheet.getAttribute('href')), sheet.type, _this.entry + '.css');
          if (!style) continue;
          sheet.replaceWith(style);
          if (style.parentNode === fragment) CSS.push(style);
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return != null) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      fragment.querySelector('template').content.prepend(...CSS);
      const script = fragment.querySelector('script');
      if (script) script.replaceWith(Object.assign(document.createElement('script'), {
        text: `\n${Component.packJS((0, _path.join)(_this.path, script.getAttribute('src')))}\n`
      }));
      return fragment;
    })();
  }
  /**
   * @protected
   *
   * @param {String} file - File path
   *
   * @return {String} Legal ECMAScript source code
   */


  assetOf(file) {
    var _this2 = this;

    return (0, _asyncToGenerator2.default)(function* () {
      switch ((0, _path.extname)(file).slice(1)) {
        case 'html':
        case 'htm':
          file = (0, _domRenderer.stringifyDOM)((yield _this2.toHTML()));
          break;

        case 'css':
        case 'less':
        case 'sass':
        case 'scss':
        case 'stylus':
          file = (yield Component.parseCSS(file)).textContent;
          break;

        case 'js':
          return;

        case 'json':
          return (yield (0, _fsExtra.readFile)(file)) + '';

        case 'yaml':
        case 'yml':
          return Parser.yaml((yield (0, _fsExtra.readFile)(file)) + '');

        default:
          file = (0, _nodeToolkit.toDataURI)(file);
      }

      return JSON.stringify(file);
    })();
  }
  /**
   * @return {string} JS version bundle of this component
   */


  toJS() {
    var _this3 = this;

    return (0, _asyncToGenerator2.default)(function* () {
      const temp_file = [];
      var _iteratorNormalCompletion2 = true;
      var _didIteratorError2 = false;
      var _iteratorError2 = undefined;

      try {
        for (var _iterator2 = (yield (0, _fsExtra.readdir)(_this3.path))[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
          let file = _step2.value;
          file = (0, _path.join)(_this3.path, file);
          if (!(0, _fsExtra.statSync)(file).isFile()) continue;
          let temp = `${file}.js`;
          file = yield _this3.assetOf(file);
          if (!(file != null)) continue;
          temp_file.push(temp);
          yield (0, _fsExtra.outputFile)(temp, `export default ${file}`);
        }
      } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion2 && _iterator2.return != null) {
            _iterator2.return();
          }
        } finally {
          if (_didIteratorError2) {
            throw _iteratorError2;
          }
        }
      }

      const source = Component.packJS(_this3.entry + '.js');
      yield Promise.all(temp_file.map(file => (0, _fsExtra.remove)(file)));
      return source;
    })();
  }

}

exports.default = Component;
//# sourceMappingURL=Component.js.map