"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.create = create;
exports.bundle = bundle;
exports.pack = pack;

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _path = require("path");

var _fsExtra = require("fs-extra");

var _Component = _interopRequireDefault(require("./Component"));

var _template = require("./template");

/**
 * @param {String}   name      - Tag name
 * @param {String}   path      - Root of Source codes
 * @param {String[]} [keys=[]] - Defined HTML attributes
 */
function create(_x, _x2, _x3) {
  return _create.apply(this, arguments);
}
/**
 * Bundle components to JS modules
 *
 * @param {string} path - Source directory
 *
 * @return {string[]} Component paths
 */


function _create() {
  _create = (0, _asyncToGenerator2.default)(function* (name, path, keys) {
    yield (0, _fsExtra.outputFile)((0, _path.join)(path, name, 'index.html'), (0, _template.index_html)());
    console.info(`√ Generated ${name}/index.html`);
    yield (0, _fsExtra.outputFile)((0, _path.join)(path, name, 'index.js'), (0, _template.index_js)(name, keys));
    console.info(`√ Generated ${name}/index.js`);
  });
  return _create.apply(this, arguments);
}

function bundle(_x4) {
  return _bundle.apply(this, arguments);
}
/**
 * Bundle components into a JS package
 *
 * @param {string} path - Source directory
 *
 * @return {string[]} Component paths
 */


function _bundle() {
  _bundle = (0, _asyncToGenerator2.default)(function* (path) {
    var result = [];

    if ((0, _fsExtra.existsSync)((0, _path.join)(path, 'index.js'))) {
      const component = new _Component.default(path);
      result[0] = `dist/${component.name}.js`;
      yield (0, _fsExtra.outputFile)(result[0], (yield component.toJS()));
    }

    if ((0, _fsExtra.statSync)(path).isDirectory()) result = result.concat(...(yield Promise.all((yield (0, _fsExtra.readdir)(path)).map(file => bundle((0, _path.join)(path, file))))));
    return result;
  });
  return _bundle.apply(this, arguments);
}

function pack(_x5) {
  return _pack.apply(this, arguments);
}

function _pack() {
  _pack = (0, _asyncToGenerator2.default)(function* (path) {
    const file = yield bundle(path);
    yield (0, _fsExtra.outputFile)('dist/index.js', file.map(item => {
      item = (0, _path.basename)(item);
      console.info(`√ Component "${item}" is packed in`);
      return `export * from './${item}';`;
    }).join('\n'));
    return file;
  });
  return _pack.apply(this, arguments);
}
//# sourceMappingURL=command.js.map